@@ -7,7 +7,7 @@ use openpgp::parse::Parse;
 use crate::common::encrypt;
 
 lazy_static::lazy_static! {
-    static ref ZEROS_1_MB: Vec<u8> = vec![0; 1 * 1024 * 1024];
+    static ref ZEROS_1_MB: Vec<u8> = vec![0; 1024 * 1024];
     static ref ZEROS_10_MB: Vec<u8> = vec![0; 10 * 1024 * 1024];
 }
 
@@ -34,12 +34,12 @@ fn bench_encrypt(c: &mut Criterion) {
         group.bench_with_input(
             BenchmarkId::new("password", message.len()),
             &message,
-            |b, m| b.iter(|| encrypt_with_password(&m)),
+            |b, m| b.iter(|| encrypt_with_password(m)),
         );
         group.bench_with_input(
             BenchmarkId::new("cert", message.len()),
             &message,
-            |b, m| b.iter(|| encrypt_to_testy(&m)),
+            |b, m| b.iter(|| encrypt_to_testy(m)),
         );
     }
     group.finish();
