@@ -8,12 +8,45 @@ use crate::{
     KeyHandle,
     types::RevocationStatus,
     packet::key,
-    packet::key::SecretKeyMaterial,
     types::KeyFlags,
     cert::prelude::*,
     policy::Policy,
 };
 
+/// Evaluates whether to skip the use of a secret key when filtering.
+///
+/// This function takes three optional boolean values, indicating whether
+/// an encrypted secret is present,
+/// whether to filter for encrypted secret keys
+/// and whether to filter for unencrypted secret keys.
+///
+/// An optional string provides context in the case where a key is skipped.
+fn skip_secret(
+    have_secret_encrypted: Option<bool>,
+    want_encrypted_key: Option<bool>,
+    want_unencrypted_secret: Option<bool>,
+) -> Option<&'static str> {
+    match (have_secret_encrypted, want_encrypted_key, want_unencrypted_secret) {
+        (Some(_), Some(true), Some(true))
+        | (_, None, None)
+        | (Some(true), Some(true), _)
+        | (Some(true), None, Some(false))
+        | (Some(false), _, Some(true))
+        | (Some(false), Some(false), None) => None,  // do not skip the key
+        (Some(true), Some(false), _) | (Some(true), None, Some(true)) => {
+            Some("Encrypted secret... skipping.")
+        }
+        (Some(false), _, Some(false)) | (Some(false), Some(true), None) => {
+            Some("Unencrypted secret... skipping.")
+        }
+        (None, Some(_), None)
+        | (None, None, Some(_))
+        | (None, Some(_), Some(_)) => {
+            Some("No secret... skipping.")
+        }
+    }
+}
+
 /// An iterator over `Key`s.
 ///
 /// An iterator over [`KeyAmalgamation`]s.
@@ -32,7 +65,8 @@ use crate::{
 ///
 /// `KeyAmalgamationIter` supports other filters.  For instance
 /// [`KeyAmalgamationIter::secret`] filters on whether secret key
-/// material is present, and
+/// material is present, [`KeyAmalgamationIter::encrypted_secret`]
+/// filters on whether secret key material is present and encrypted, and
 /// [`KeyAmalgamationIter::unencrypted_secret`] filters on whether
 /// secret key material is present and unencrypted.  Of course, since
 /// `KeyAmalgamationIter` implements `Iterator`, it is possible to use
@@ -61,8 +95,8 @@ pub struct KeyAmalgamationIter<'a, P, R>
     subkey_iter: slice::Iter<'a, KeyBundle<key::PublicParts,
                                            key::SubordinateRole>>,
 
-    // If not None, filters by whether a key has a secret.
-    secret: Option<bool>,
+    // If not None, filters by whether a key has an encrypted secret.
+    encrypted_secret: Option<bool>,
 
     // If not None, filters by whether a key has an unencrypted
     // secret.
@@ -89,7 +123,7 @@ impl<'a, P, R> fmt::Debug for KeyAmalgamationIter<'a, P, R>
 {
     fn fmt(&self, f: &mut fmt::Formatter) -> fmt::Result {
         f.debug_struct("KeyAmalgamationIter")
-            .field("secret", &self.secret)
+            .field("encrypted_secret", &self.encrypted_secret)
             .field("unencrypted_secret", &self.unencrypted_secret)
             .field("key_handles", &self.key_handles)
             .field("supported", &self.supported)
@@ -181,35 +215,13 @@ impl<'a, P, R> KeyAmalgamationIter<'a, P, R>
                 }
             }
 
-            if let Some(want_secret) = self.secret {
-                if ka.key().has_secret() {
-                    // We have a secret.
-                    if ! want_secret {
-                        t!("Have a secret... skipping.");
-                        continue;
-                    }
-                } else if want_secret {
-                    t!("No secret... skipping.");
-                    continue;
-                }
-            }
-
-            if let Some(want_unencrypted_secret) = self.unencrypted_secret {
-                if let Some(secret) = ka.key().optional_secret() {
-                    if let SecretKeyMaterial::Unencrypted { .. } = secret {
-                        if ! want_unencrypted_secret {
-                            t!("Unencrypted secret... skipping.");
-                            continue;
-                        }
-                    } else if want_unencrypted_secret {
-                        t!("Encrypted secret... skipping.");
-                        continue;
-                    }
-                } else {
-                    // No secret.
-                    t!("No secret... skipping.");
-                    continue;
-                }
+            if let Some(msg) = skip_secret(
+                ka.key().optional_secret().map(|x| x.is_encrypted()),
+                self.encrypted_secret,
+                self.unencrypted_secret,
+            ) {
+                t!(msg);
+                continue;
             }
 
             return Some(ka);
@@ -229,7 +241,7 @@ impl<'a, P, R> KeyAmalgamationIter<'a, P, R>
             subkey_iter: cert.subkeys.iter(),
 
             // The filters.
-            secret: None,
+            encrypted_secret: None,
             unencrypted_secret: None,
             key_handles: None,
             supported: None,
@@ -250,11 +262,15 @@ impl<'a, P, R> KeyAmalgamationIter<'a, P, R>
     /// # use openpgp::cert::prelude::*;
     /// # fn main() -> Result<()> {
     /// #     let (cert, _) =
-    /// #         CertBuilder::general_purpose(None, Some("alice@example.org"))
+    /// #         CertBuilder::new().set_password(Some("password".into()))
     /// #         .generate()?;
     /// for ka in cert.keys().secret() {
     ///     // Use it.
     /// }
+    /// #     assert!(cert.keys().secret().count() == 1);
+    /// #
+    /// #     let (cert, _) = CertBuilder::new().generate()?;
+    /// #     assert!(cert.keys().secret().count() == 1);
     /// #     Ok(())
     /// # }
     /// ```
@@ -265,7 +281,46 @@ impl<'a, P, R> KeyAmalgamationIter<'a, P, R>
             subkey_iter: self.subkey_iter,
 
             // The filters.
-            secret: Some(true),
+            encrypted_secret: Some(true),
+            unencrypted_secret: Some(true),
+            key_handles: self.key_handles,
+            supported: self.supported,
+
+            _p: std::marker::PhantomData,
+            _r: std::marker::PhantomData,
+        }
+    }
+
+    /// Changes the iterator to only return keys with encrypted secret key
+    /// material.
+    ///
+    /// # Examples
+    ///
+    /// ```rust
+    /// # use sequoia_openpgp as openpgp;
+    /// # use openpgp::Result;
+    /// # use openpgp::cert::prelude::*;
+    /// # fn main() -> Result<()> {
+    /// #     let (cert, _) =
+    /// #         CertBuilder::new().set_password(Some("password".into()))
+    /// #         .generate()?;
+    /// for ka in cert.keys().encrypted_secret() {
+    ///     // Use it.
+    /// }
+    /// #     assert!(cert.keys().encrypted_secret().count() == 1);
+    /// #     Ok(())
+    /// # }
+    /// ```
+    pub fn encrypted_secret(
+        self,
+    ) -> KeyAmalgamationIter<'a, key::SecretParts, R> {
+        KeyAmalgamationIter {
+            cert: self.cert,
+            primary: self.primary,
+            subkey_iter: self.subkey_iter,
+
+            // The filters.
+            encrypted_secret: Some(true),
             unencrypted_secret: self.unencrypted_secret,
             key_handles: self.key_handles,
             supported: self.supported,
@@ -286,11 +341,11 @@ impl<'a, P, R> KeyAmalgamationIter<'a, P, R>
     /// # use openpgp::cert::prelude::*;
     /// # fn main() -> Result<()> {
     /// #     let (cert, _) =
-    /// #         CertBuilder::general_purpose(None, Some("alice@example.org"))
-    /// #         .generate()?;
+    /// #         CertBuilder::new().generate()?;
     /// for ka in cert.keys().unencrypted_secret() {
     ///     // Use it.
     /// }
+    /// #     assert!(cert.keys().secret().unencrypted_secret().count() == 1);
     /// #     Ok(())
     /// # }
     /// ```
@@ -301,7 +356,7 @@ impl<'a, P, R> KeyAmalgamationIter<'a, P, R>
             subkey_iter: self.subkey_iter,
 
             // The filters.
-            secret: self.secret,
+            encrypted_secret: self.encrypted_secret,
             unencrypted_secret: Some(true),
             key_handles: self.key_handles,
             supported: self.supported,
@@ -468,7 +523,7 @@ impl<'a, P, R> KeyAmalgamationIter<'a, P, R>
             subkey_iter: self.subkey_iter,
 
             // The filters.
-            secret: self.secret,
+            encrypted_secret: self.encrypted_secret,
             unencrypted_secret: self.unencrypted_secret,
             key_handles: self.key_handles,
             supported: self.supported,
@@ -527,7 +582,7 @@ impl<'a, P, R> KeyAmalgamationIter<'a, P, R>
             time: time.into().unwrap_or_else(crate::now),
 
             // The filters.
-            secret: self.secret,
+            encrypted_secret: self.encrypted_secret,
             unencrypted_secret: self.unencrypted_secret,
             key_handles: self.key_handles,
             supported: self.supported,
@@ -635,8 +690,8 @@ pub struct ValidKeyAmalgamationIter<'a, P, R>
     // The time.
     time: SystemTime,
 
-    // If not None, filters by whether a key has a secret.
-    secret: Option<bool>,
+    // If not None, filters by whether a key has an encrypted secret.
+    encrypted_secret: Option<bool>,
 
     // If not None, filters by whether a key has an unencrypted
     // secret.
@@ -675,7 +730,7 @@ impl<'a, P, R> fmt::Debug for ValidKeyAmalgamationIter<'a, P, R>
         f.debug_struct("ValidKeyAmalgamationIter")
             .field("policy", &self.policy)
             .field("time", &self.time)
-            .field("secret", &self.secret)
+            .field("encrypted_secret", &self.encrypted_secret)
             .field("unencrypted_secret", &self.unencrypted_secret)
             .field("key_handles", &self.key_handles)
             .field("supported", &self.supported)
@@ -800,7 +855,7 @@ impl<'a, P, R> ValidKeyAmalgamationIter<'a, P, R>
             if let Some(flags) = self.flags.as_ref() {
                 if !ka.has_any_key_flag(flags) {
                     t!("Have flags: {:?}, want flags: {:?}... skipping.",
-                      flags, flags);
+                       ka.key_flags(), flags);
                     continue;
                 }
             }
@@ -828,35 +883,13 @@ impl<'a, P, R> ValidKeyAmalgamationIter<'a, P, R>
                 }
             }
 
-            if let Some(want_secret) = self.secret {
-                if key.has_secret() {
-                    // We have a secret.
-                    if ! want_secret {
-                        t!("Have a secret... skipping.");
-                        continue;
-                    }
-                } else if want_secret {
-                    t!("No secret... skipping.");
-                    continue;
-                }
-            }
-
-            if let Some(want_unencrypted_secret) = self.unencrypted_secret {
-                if let Some(secret) = key.optional_secret() {
-                    if let SecretKeyMaterial::Unencrypted { .. } = secret {
-                        if ! want_unencrypted_secret {
-                            t!("Unencrypted secret... skipping.");
-                            continue;
-                        }
-                    } else if want_unencrypted_secret {
-                        t!("Encrypted secret... skipping.");
-                        continue;
-                    }
-                } else {
-                    // No secret.
-                    t!("No secret... skipping.");
-                    continue;
-                }
+            if let Some(msg) = skip_secret(
+                ka.key().optional_secret().map(|x| x.is_encrypted()),
+                self.encrypted_secret,
+                self.unencrypted_secret,
+            ) {
+                t!(msg);
+                continue;
             }
 
             return Some(ka);
@@ -1328,11 +1361,15 @@ impl<'a, P, R> ValidKeyAmalgamationIter<'a, P, R>
     /// let p = &StandardPolicy::new();
     ///
     /// #     let (cert, _) =
-    /// #         CertBuilder::general_purpose(None, Some("alice@example.org"))
+    /// #         CertBuilder::new().set_password(Some("password".into()))
     /// #         .generate()?;
     /// for ka in cert.keys().with_policy(p, None).secret() {
     ///     // Use it.
     /// }
+    /// #     assert!(cert.keys().with_policy(p, None).secret().count() == 1);
+    /// #
+    /// #     let (cert, _) = CertBuilder::new().generate()?;
+    /// #     assert!(cert.keys().with_policy(p, None).secret().count() == 1);
     /// #     Ok(())
     /// # }
     /// ```
@@ -1346,7 +1383,56 @@ impl<'a, P, R> ValidKeyAmalgamationIter<'a, P, R>
             policy: self.policy,
 
             // The filters.
-            secret: Some(true),
+            encrypted_secret: Some(true),
+            unencrypted_secret: Some(true),
+            key_handles: self.key_handles,
+            supported: self.supported,
+            flags: self.flags,
+            alive: self.alive,
+            revoked: self.revoked,
+
+            _p: std::marker::PhantomData,
+            _r: std::marker::PhantomData,
+        }
+    }
+
+    /// Changes the iterator to only return keys with encrypted secret key
+    /// material.
+    ///
+    /// # Examples
+    ///
+    /// ```rust
+    /// # use sequoia_openpgp as openpgp;
+    /// # use openpgp::Result;
+    /// # use openpgp::cert::prelude::*;
+    /// use openpgp::policy::StandardPolicy;
+    ///
+    /// # fn main() -> Result<()> {
+    /// let p = &StandardPolicy::new();
+    ///
+    /// #     let (cert, _) =
+    /// #         CertBuilder::new().set_password(Some("password".into()))
+    /// #         .generate()?;
+    /// for ka in cert.keys().with_policy(p, None).encrypted_secret() {
+    ///     // Use it.
+    /// }
+    /// #     assert!(cert.keys().with_policy(p, None).encrypted_secret().count() == 1);
+    /// #     Ok(())
+    /// # }
+    /// ```
+    pub fn encrypted_secret(
+        self,
+    ) -> ValidKeyAmalgamationIter<'a, key::SecretParts, R> {
+        ValidKeyAmalgamationIter {
+            cert: self.cert,
+            primary: self.primary,
+            subkey_iter: self.subkey_iter,
+
+            time: self.time,
+            policy: self.policy,
+
+            // The filters.
+            encrypted_secret: Some(true),
             unencrypted_secret: self.unencrypted_secret,
             key_handles: self.key_handles,
             supported: self.supported,
@@ -1373,12 +1459,11 @@ impl<'a, P, R> ValidKeyAmalgamationIter<'a, P, R>
     /// # fn main() -> Result<()> {
     /// let p = &StandardPolicy::new();
     ///
-    /// #     let (cert, _) =
-    /// #         CertBuilder::general_purpose(None, Some("alice@example.org"))
-    /// #         .generate()?;
+    /// #     let (cert, _) = CertBuilder::new().generate()?;
     /// for ka in cert.keys().with_policy(p, None).unencrypted_secret() {
     ///     // Use it.
     /// }
+    /// #     assert!(cert.keys().with_policy(p, None).unencrypted_secret().count() == 1);
     /// #     Ok(())
     /// # }
     /// ```
@@ -1392,7 +1477,7 @@ impl<'a, P, R> ValidKeyAmalgamationIter<'a, P, R>
             policy: self.policy,
 
             // The filters.
-            secret: self.secret,
+            encrypted_secret: self.encrypted_secret,
             unencrypted_secret: Some(true),
             key_handles: self.key_handles,
             supported: self.supported,
@@ -1578,7 +1663,7 @@ impl<'a, P, R> ValidKeyAmalgamationIter<'a, P, R>
             policy: self.policy,
 
             // The filters.
-            secret: self.secret,
+            encrypted_secret: self.encrypted_secret,
             unencrypted_secret: self.unencrypted_secret,
             key_handles: self.key_handles,
             supported: self.supported,
