@@ -620,7 +620,6 @@ impl<'a> Parse<'a, CertParser<'a>> for CertParser<'a>
     }
 }
 
-#[allow(clippy::should_implement_trait)]
 impl<'a> CertParser<'a> {
     /// Creates a `CertParser` from a `Result<Packet>` iterator.
     ///
@@ -754,8 +753,8 @@ impl<'a> CertParser<'a> {
     /// #     Ok(())
     /// # }
     /// ```
-    pub fn unvalidated_cert_filter<F: 'a>(mut self, filter: F) -> Self
-        where F: Send + Sync + Fn(&Cert, bool) -> bool
+    pub fn unvalidated_cert_filter<F>(mut self, filter: F) -> Self
+        where F: 'a + Send + Sync + Fn(&Cert, bool) -> bool
     {
         self.filter.push(Box::new(filter));
         self
