@@ -10,43 +10,22 @@ pub mod ecdh;
 pub mod hash;
 pub mod symmetric;
 
-/// Returns a short, human-readable description of the backend.
-pub fn backend() -> String {
-    let (major, minor) = nettle::version();
-    format!(
-        "Nettle {}.{} (Cv448: {:?})",
-        major, minor,
-        nettle::curve448::IS_SUPPORTED,
-    )
-}
+pub struct Backend(());
 
-/// Fills the given buffer with random data.
-pub fn random(buf: &mut [u8]) {
-    Yarrow::default().random(buf);
-}
-
-impl PublicKeyAlgorithm {
-    pub(crate) fn is_supported_by_backend(&self) -> bool {
-        use PublicKeyAlgorithm::*;
-        #[allow(deprecated)]
-        match &self {
-            RSAEncryptSign | RSAEncrypt | RSASign | DSA | ECDH | ECDSA | EdDSA
-                => true,
-            ElGamalEncrypt | ElGamalEncryptSign | Private(_) | Unknown(_)
-                => false,
-        }
+impl super::interface::Backend for Backend {
+    fn backend() -> String {
+        let (major, minor) = nettle::version();
+        format!(
+            "Nettle {}.{} (Cv448: {:?}, OCB: {:?})",
+            major, minor,
+            nettle::curve448::IS_SUPPORTED,
+            nettle::aead::OCB_IS_SUPPORTED,
+        )
     }
-}
 
-impl Curve {
-    pub(crate) fn is_supported_by_backend(&self) -> bool {
-        use self::Curve::*;
-        match &self {
-            NistP256 | NistP384 | NistP521 | Ed25519 | Cv25519
-                => true,
-            BrainpoolP256 | BrainpoolP512 | Unknown(_)
-                => false,
-        }
+    fn random(buf: &mut [u8]) -> crate::Result<()> {
+        Yarrow::default().random(buf);
+        Ok(())
     }
 }
 
@@ -57,7 +36,11 @@ impl AEADAlgorithm {
     /// algorithm and the most performing one, but fall back to any
     /// supported algorithm.
     pub(crate) const fn const_default() -> AEADAlgorithm {
-        AEADAlgorithm::EAX
+        if nettle::aead::OCB_IS_SUPPORTED {
+            AEADAlgorithm::OCB
+        } else {
+            AEADAlgorithm::EAX
+        }
     }
 
     pub(crate) fn is_supported_by_backend(&self) -> bool {
@@ -65,7 +48,11 @@ impl AEADAlgorithm {
         match &self {
             EAX
                 => true,
-            OCB | Private(_) | Unknown(_)
+            OCB
+                => nettle::aead::OCB_IS_SUPPORTED,
+            GCM
+                => true,
+            Private(_) | Unknown(_)
                 => false,
         }
     }
@@ -84,6 +71,28 @@ impl AEADAlgorithm {
                     SymmetricAlgorithm::Camellia256 => true,
                     _ => false,
                 },
+            AEADAlgorithm::OCB =>
+                match algo {
+                    SymmetricAlgorithm::AES128 |
+                    SymmetricAlgorithm::AES192 |
+                    SymmetricAlgorithm::AES256 |
+                    SymmetricAlgorithm::Twofish |
+                    SymmetricAlgorithm::Camellia128 |
+                    SymmetricAlgorithm::Camellia192 |
+                    SymmetricAlgorithm::Camellia256 => true,
+                    _ => false,
+                },
+            AEADAlgorithm::GCM =>
+                match algo {
+                    SymmetricAlgorithm::AES128 |
+                    SymmetricAlgorithm::AES192 |
+                    SymmetricAlgorithm::AES256 |
+                    SymmetricAlgorithm::Twofish |
+                    SymmetricAlgorithm::Camellia128 |
+                    SymmetricAlgorithm::Camellia192 |
+                    SymmetricAlgorithm::Camellia256 => true,
+                    _ => false,
+                },
             _ => false
         }
     }
