@@ -1,7 +1,7 @@
 //! Implementation of Sequoia crypto API using pure Rust cryptographic
 //! libraries.
 
-use generic_array::{ArrayLength, GenericArray};
+use cipher::generic_array::{ArrayLength, GenericArray};
 
 use crate::{Error, Result};
 use crate::types::*;
@@ -12,6 +12,22 @@ pub mod ecdh;
 pub mod hash;
 pub mod symmetric;
 
+pub struct Backend(());
+
+impl super::interface::Backend for Backend {
+    fn backend() -> String {
+        // XXX: can we include features and the version?
+        "RustCrypto".to_string()
+    }
+
+    fn random(buf: &mut [u8]) -> Result<()> {
+        use rand::rngs::OsRng;
+        use rand::RngCore;
+        OsRng.fill_bytes(buf);
+        Ok(())
+    }
+}
+
 trait GenericArrayExt<T, N: ArrayLength<T>> {
     const LEN: usize;
 
@@ -44,51 +60,6 @@ impl<T, N: ArrayLength<T>> GenericArrayExt<T, N> for GenericArray<T, N> {
     const LEN: usize = N::USIZE;
 }
 
-/// Returns a short, human-readable description of the backend.
-pub fn backend() -> String {
-    // XXX: can we include features and the version?
-    "RustCrypto".to_string()
-}
-
-/// Fills the given buffer with random data.
-pub fn random(buf: &mut [u8]) {
-    use rand07::rngs::OsRng;
-    use rand07::RngCore;
-
-    OsRng.fill_bytes(buf)
-}
-
-impl PublicKeyAlgorithm {
-    pub(crate) fn is_supported_by_backend(&self) -> bool {
-        use PublicKeyAlgorithm::*;
-        #[allow(deprecated)]
-        match &self {
-            RSAEncryptSign | RSAEncrypt | RSASign | ECDH | EdDSA | ECDSA
-                => true,
-            DSA
-                => false,
-            ElGamalEncrypt | ElGamalEncryptSign | Private(_) | Unknown(_)
-                => false,
-        }
-    }
-}
-
-impl Curve {
-    pub(crate) fn is_supported_by_backend(&self) -> bool {
-        use self::Curve::*;
-        match &self {
-            NistP256
-                => true,
-            NistP384 | NistP521
-                => false,
-            Ed25519 | Cv25519
-                => true,
-            BrainpoolP256 | BrainpoolP512 | Unknown(_)
-                => false,
-        }
-    }
-}
-
 impl AEADAlgorithm {
     /// Returns the best AEAD mode supported by the backend.
     ///
@@ -102,9 +73,10 @@ impl AEADAlgorithm {
     pub(crate) fn is_supported_by_backend(&self) -> bool {
         use self::AEADAlgorithm::*;
         match &self {
-            EAX
-                => true,
-            OCB | Private(_) | Unknown(_)
+            EAX => true,
+            OCB => false,
+            GCM => true,
+            Private(_) | Unknown(_)
                 => false,
         }
     }
@@ -124,6 +96,14 @@ impl AEADAlgorithm {
                     SymmetricAlgorithm::Camellia256 => true,
                     _ => false,
                 },
+
+            AEADAlgorithm::GCM =>
+                match algo {
+                    SymmetricAlgorithm::AES128 |
+                    SymmetricAlgorithm::AES192 |
+                    SymmetricAlgorithm::AES256 => true,
+                    _ => false,
+                },
             _ => false
         }
     }
