@@ -305,13 +305,13 @@ impl Arbitrary for MPI {
 
 impl PartialOrd for MPI {
     fn partial_cmp(&self, other: &MPI) -> Option<Ordering> {
-        Some(self.secure_memcmp(other))
+        Some(self.cmp(other))
     }
 }
 
 impl Ord for MPI {
     fn cmp(&self, other: &MPI) -> Ordering {
-        self.partial_cmp(other).unwrap()
+        self.secure_memcmp(other)
     }
 }
 
@@ -392,13 +392,13 @@ impl From<MPI> for ProtectedMPI {
 
 impl PartialOrd for ProtectedMPI {
     fn partial_cmp(&self, other: &ProtectedMPI) -> Option<Ordering> {
-        Some(self.secure_memcmp(other))
+        Some(self.cmp(other))
     }
 }
 
 impl Ord for ProtectedMPI {
     fn cmp(&self, other: &ProtectedMPI) -> Ordering {
-        self.partial_cmp(other).unwrap()
+        self.secure_memcmp(other)
     }
 }
 
@@ -700,8 +700,8 @@ impl Arbitrary for PublicKey {
 /// extensions.
 // Deriving Hash here is okay: PartialEq is manually implemented to
 // ensure that secrets are compared in constant-time.
-#[allow(clippy::derived_hash_with_manual_eq)]
 #[non_exhaustive]
+#[allow(clippy::derived_hash_with_manual_eq)]
 #[derive(Clone, Hash)]
 pub enum SecretKeyMaterial {
     /// RSA secret key.
@@ -798,6 +798,12 @@ impl fmt::Debug for SecretKeyMaterial {
 
 impl PartialOrd for SecretKeyMaterial {
     fn partial_cmp(&self, other: &SecretKeyMaterial) -> Option<Ordering> {
+        Some(self.cmp(other))
+    }
+}
+
+impl Ord for SecretKeyMaterial {
+    fn cmp(&self, other: &Self) -> Ordering {
         use std::iter;
 
         fn discriminant(sk: &SecretKeyMaterial) -> usize {
@@ -820,9 +826,9 @@ impl PartialOrd for SecretKeyMaterial {
                 let o3 = q1.cmp(q2);
                 let o4 = u1.cmp(u2);
 
-                if o1 != Ordering::Equal { return Some(o1); }
-                if o2 != Ordering::Equal { return Some(o2); }
-                if o3 != Ordering::Equal { return Some(o3); }
+                if o1 != Ordering::Equal { return o1; }
+                if o2 != Ordering::Equal { return o2; }
+                if o3 != Ordering::Equal { return o3; }
                 o4
             }
             (&SecretKeyMaterial::DSA{ x: ref x1 }
@@ -848,11 +854,13 @@ impl PartialOrd for SecretKeyMaterial {
             (&SecretKeyMaterial::Unknown{ mpis: ref mpis1, rest: ref rest1 }
             ,&SecretKeyMaterial::Unknown{ mpis: ref mpis2, rest: ref rest2 }) => {
                 let o1 = secure_cmp(rest1, rest2);
+                let o2 = mpis1.len().cmp(&mpis2.len());
                 let on = mpis1.iter().zip(mpis2.iter()).map(|(a,b)| {
                     a.cmp(b)
                 }).collect::<Vec<_>>();
 
                 iter::once(o1)
+                    .chain(iter::once(o2))
                     .chain(on.iter().cloned())
                     .fold(Ordering::Equal, |acc, x| acc.then(x))
             }
@@ -865,13 +873,7 @@ impl PartialOrd for SecretKeyMaterial {
             }
         };
 
-        Some(ret)
-    }
-}
-
-impl Ord for SecretKeyMaterial {
-    fn cmp(&self, other: &Self) -> Ordering {
-        self.partial_cmp(other).unwrap()
+        ret
     }
 }
 
