@@ -60,14 +60,14 @@ impl<'a> Decryptor<'a> {
     where
         R: io::Read + Send + Sync + 'a,
     {
-        Self::from_buffered_reader(
+        Self::from_cookie_reader(
             algo, key,
             Box::new(buffered_reader::Generic::with_cookie(
                 source, None, Default::default())))
     }
 
     /// Instantiate a new symmetric decryptor.
-    fn from_buffered_reader(algo: SymmetricAlgorithm, key: &[u8],
+    fn from_cookie_reader(algo: SymmetricAlgorithm, key: &[u8],
                             source: Box<dyn BufferedReader<Cookie> + 'a>)
                             -> Result<Self>
     {
@@ -182,7 +182,7 @@ impl<'a> BufferedReaderDecryptor<'a> {
     {
         Ok(BufferedReaderDecryptor {
             reader: buffered_reader::Generic::with_cookie(
-                Decryptor::from_buffered_reader(algo, key, reader)?,
+                Decryptor::from_cookie_reader(algo, key, reader)?,
                 None, cookie),
         })
     }
@@ -264,7 +264,7 @@ impl<'a> BufferedReader<Cookie> for BufferedReaderDecryptor<'a> {
 
     fn into_inner<'b>(self: Box<Self>)
             -> Option<Box<dyn BufferedReader<Cookie> + 'b>> where Self: 'b {
-        Some(self.reader.into_reader().source.as_boxed())
+        Some(self.reader.into_reader().source.into_boxed())
     }
 
     fn cookie_set(&mut self, cookie: Cookie) -> Cookie {
