@@ -20,6 +20,7 @@ use crate::Result;
 macro_rules! convert {
     ( $x:ident ) => {
         // XXX: This is ugly, but how can we do better?
+        #[allow(clippy::missing_transmute_annotations)]
         unsafe { std::mem::transmute($x) }
     }
 }
@@ -27,7 +28,7 @@ macro_rules! convert {
 macro_rules! convert_ref {
     ( $x:ident ) => {
         // XXX: This is ugly, but how can we do better?
-        #[allow(clippy::transmute_ptr_to_ptr)]
+        #[allow(clippy::missing_transmute_annotations)]
         unsafe { std::mem::transmute($x) }
     }
 }
@@ -249,12 +250,12 @@ macro_rules! create_part_conversions {
             }
 
             /// Changes the key's parts tag to `UnspecifiedParts`.
-            pub fn parts_as_unspecified(&$($l)* self) -> &$Key<$($l, )* UnspecifiedParts, $($g, )*> {
+            pub fn parts_as_unspecified(&$($l)* self) -> &$($l)* $Key<$($l, )* UnspecifiedParts, $($g, )*> {
                 convert_ref!(self)
             }
 
             /// Changes the key's parts tag to `UnspecifiedParts`.
-            pub fn parts_as_unspecified_mut(&$($l)* mut self) -> &mut $Key<$($l, )* UnspecifiedParts, $($g, )*> {
+            pub fn parts_as_unspecified_mut(&$($l)* mut self) -> &$($l)* mut $Key<$($l, )* UnspecifiedParts, $($g, )*> {
                 convert_ref!(self)
             }
         }
