@@ -640,8 +640,8 @@ impl PartialEq for Common {
 impl Eq for Common {}
 
 impl PartialOrd for Common {
-    fn partial_cmp(&self, _: &Self) -> Option<std::cmp::Ordering> {
-        Some(std::cmp::Ordering::Equal)
+    fn partial_cmp(&self, other: &Self) -> Option<std::cmp::Ordering> {
+        Some(self.cmp(other))
     }
 }
 
