@@ -56,7 +56,7 @@ impl PKESK3 {
         })
     }
 
-    /// Creates a new PKESK3 packet for the given recipent.
+    /// Creates a new PKESK3 packet for the given recipient.
     ///
     /// The given symmetric algorithm must match the algorithm that is
     /// used to encrypt the payload.
@@ -309,7 +309,8 @@ mod tests {
         let pkesk: &PKESK =
             pile.descendants().next().unwrap().downcast_ref().unwrap();
 
-        let plain = pkesk.decrypt(&mut keypair, None).unwrap();
+        let plain = pkesk.decrypt(&mut keypair, None)
+            .expect("ECDH decryption using P-256 key should work");
         let plain_ =
             pkesk.decrypt(&mut keypair, Some(SymmetricAlgorithm::AES256))
             .unwrap();
@@ -338,6 +339,35 @@ mod tests {
         let pkesk: &PKESK =
             pile.descendants().next().unwrap().downcast_ref().unwrap();
 
+        let plain = pkesk.decrypt(&mut keypair, None)
+            .expect("ECDH decryption using P-384 key should work");
+        let plain_ =
+            pkesk.decrypt(&mut keypair, Some(SymmetricAlgorithm::AES256))
+            .unwrap();
+        assert_eq!(plain, plain_);
+
+        eprintln!("plain: {:?}", plain);
+    }
+
+    #[test]
+    fn decrypt_elgamal() -> Result<()> {
+        if ! (PublicKeyAlgorithm::DSA.is_supported()
+              && PublicKeyAlgorithm::ElGamalEncrypt.is_supported()) {
+            eprintln!("Skipping test, algorithm is not supported.");
+            return Ok(());
+        }
+
+        let cert = Cert::from_bytes(
+            crate::tests::key("dsa2048-elgamal3072-private.pgp"))?;
+        let pile = PacketPile::from_bytes(
+            crate::tests::message("encrypted-to-dsa2048-elgamal3072.pgp"))?;
+        let mut keypair =
+            cert.subkeys().next().unwrap()
+            .key().clone().parts_into_secret()?.into_keypair()?;
+
+        let pkesk: &PKESK =
+            pile.descendants().next().unwrap().downcast_ref().unwrap();
+
         let plain = pkesk.decrypt(&mut keypair, None).unwrap();
         let plain_ =
             pkesk.decrypt(&mut keypair, Some(SymmetricAlgorithm::AES256))
@@ -345,6 +375,7 @@ mod tests {
         assert_eq!(plain, plain_);
 
         eprintln!("plain: {:?}", plain);
+        Ok(())
     }
 
     #[test]
@@ -367,7 +398,8 @@ mod tests {
         let pkesk: &PKESK =
             pile.descendants().next().unwrap().downcast_ref().unwrap();
 
-        let plain = pkesk.decrypt(&mut keypair, None).unwrap();
+        let plain = pkesk.decrypt(&mut keypair, None)
+            .expect("ECDH decryption using P-521 key should work");
         let plain_ =
             pkesk.decrypt(&mut keypair, Some(SymmetricAlgorithm::AES256))
             .unwrap();
