@@ -1,13 +1,13 @@
+use std::borrow::Cow;
 use std::fmt;
 use std::str;
 use std::hash::{Hash, Hasher};
-use std::cell::RefCell;
 use std::cmp::Ordering;
-use std::sync::Mutex;
 
 #[cfg(test)]
 use quickcheck::{Arbitrary, Gen};
 
+use once_cell::sync::OnceCell;
 use regex::Regex;
 
 use crate::Result;
@@ -472,21 +472,45 @@ pub struct UserID {
     ///   [RFC 4880]: https://tools.ietf.org/html/rfc4880#section-5.11
     ///
     /// Use `UserID::default()` to get a UserID with a default settings.
-    value: Vec<u8>,
+    value: Cow<'static, [u8]>,
 
-    hash_algo_security: HashAlgoSecurity,
+    hash_algo_security: OnceCell<HashAlgoSecurity>,
 
-    parsed: Mutex<RefCell<Option<ConventionallyParsedUserID>>>,
+    parsed: OnceCell<ConventionallyParsedUserID>,
 }
 assert_send_and_sync!(UserID);
 
+impl UserID {
+    /// Returns a User ID.
+    ///
+    /// This is equivalent to using `UserID::from`, but the function
+    /// is constant, and the slice must have a static lifetime.
+    ///
+    /// # Examples
+    ///
+    /// ```
+    /// use sequoia_openpgp::packet::UserID;
+    ///
+    /// const TRUST_ROOT_USERID: UserID
+    ///     = UserID::from_static_bytes(b"Local Trust Root");
+    /// ```
+    pub const fn from_static_bytes(u: &'static [u8]) -> Self {
+        UserID {
+            common: packet::Common::new(),
+            hash_algo_security: OnceCell::new(),
+            value: Cow::Borrowed(u),
+            parsed: OnceCell::new(),
+        }
+    }
+}
+
 impl From<Vec<u8>> for UserID {
     fn from(u: Vec<u8>) -> Self {
         UserID {
             common: Default::default(),
-            hash_algo_security: UserID::determine_hash_algo_security(&u),
-            value: u,
-            parsed: Mutex::new(RefCell::new(None)),
+            hash_algo_security: Default::default(),
+            value: Cow::Owned(u),
+            parsed: Default::default(),
         }
     }
 }
@@ -499,26 +523,22 @@ impl From<&[u8]> for UserID {
 
 impl<'a> From<&'a str> for UserID {
     fn from(u: &'a str) -> Self {
-        let b = u.as_bytes();
-        let mut v = Vec::with_capacity(b.len());
-        v.extend_from_slice(b);
-        v.into()
+        u.as_bytes().into()
     }
 }
 
 impl From<String> for UserID {
     fn from(u: String) -> Self {
-        let u = &u[..];
-        u.into()
+        u.into_bytes().into()
     }
 }
 
-impl<'a> From<::std::borrow::Cow<'a, str>> for UserID {
-    fn from(u: ::std::borrow::Cow<'a, str>) -> Self {
-        let b = u.as_bytes();
-        let mut v = Vec::with_capacity(b.len());
-        v.extend_from_slice(b);
-        v.into()
+impl<'a> From<Cow<'a, str>> for UserID {
+    fn from(u: Cow<'a, str>) -> Self {
+        match u {
+            Cow::Owned(u) => u.into(),
+            Cow::Borrowed(u) => u.into(),
+        }
     }
 }
 
@@ -574,9 +594,13 @@ impl Clone for UserID {
     fn clone(&self) -> Self {
         UserID {
             common: self.common.clone(),
-            hash_algo_security: self.hash_algo_security,
+            hash_algo_security: self.hash_algo_security.clone(),
             value: self.value.clone(),
-            parsed: Mutex::new(RefCell::new(None)),
+            parsed: if let Some(p) = self.parsed.get() {
+                OnceCell::with_value(p.clone())
+            } else {
+                OnceCell::new()
+            },
         }
     }
 }
@@ -617,7 +641,7 @@ impl UserID {
                         "Validating comment ({:?})",
                         comment))),
                 Ok(p) => {
-                    if !(p.name().is_none()
+                    if !(p.name().is_some()
                          && p.comment().is_some()
                          && p.email().is_none()) {
                     return Err(Error::InvalidArgument(
@@ -654,14 +678,12 @@ impl UserID {
             }
         }
 
-        let something = !value.is_empty();
-        if something {
-            value.push_str(" <");
+        if !value.is_empty() {
+            value.push(' ');
         }
+        value.push('<');
         value.push_str(address);
-        if something {
-            value.push('>');
-        }
+        value.push('>');
 
         if check_address {
             // Make sure the combined thing is valid.
@@ -731,6 +753,10 @@ impl UserID {
     ///   [HashAlgoSecurity]: crate::policy::HashAlgoSecurity
     pub fn hash_algo_security(&self) -> HashAlgoSecurity {
         self.hash_algo_security
+            .get_or_init(|| {
+                UserID::determine_hash_algo_security(&self.value)
+            })
+            .clone()
     }
 
     // See documentation for hash_algo_security.
@@ -776,8 +802,15 @@ impl UserID {
     /// # use openpgp::packet::UserID;
     /// assert_eq!(UserID::from_address(
     ///                "John Smith".into(),
-    ///                None, "boat@example.org")?.value(),
+    ///                None,
+    ///                "boat@example.org")?.value(),
     ///            &b"John Smith <boat@example.org>"[..]);
+    ///
+    /// assert_eq!(UserID::from_address(
+    ///                "John Smith",
+    ///                "Who is Advok?",
+    ///                "boat@example.org")?.value(),
+    ///            &b"John Smith (Who is Advok?) <boat@example.org>"[..]);
     /// # Ok(()) }
     /// ```
     pub fn from_address<O, S>(name: O, comment: O, email: S)
@@ -841,24 +874,19 @@ impl UserID {
     ///   [`UserID::comment`]: UserID::comment()
     ///   [conventional User ID]: #conventional-user-ids
     pub fn value(&self) -> &[u8] {
-        self.value.as_slice()
+        &self.value
     }
 
-    fn do_parse(&self) -> Result<()> {
-        if self.parsed.lock().unwrap().borrow().is_none() {
+    fn do_parse(&self) -> Result<&ConventionallyParsedUserID> {
+        self.parsed.get_or_try_init(|| {
             let s = str::from_utf8(&self.value)?;
 
-            *self.parsed.lock().unwrap().borrow_mut() =
-              Some(match ConventionallyParsedUserID::new(s) {
-                Ok(puid) => puid,
-                Err(err) => {
-                    // Return the error from the NameAddrOrOther parser.
-                    return Err(err.context(format!(
-                        "Failed to parse User ID: {:?}", s)));
-                }
-            });
-        }
-        Ok(())
+            ConventionallyParsedUserID::new(s).map_err(|err| {
+                // Return the error from the NameAddrOrOther parser.
+                err.context(format!(
+                    "Failed to parse User ID: {:?}", s))
+            })
+        })
     }
 
     /// Parses the User ID according to de facto conventions, and
@@ -867,12 +895,17 @@ impl UserID {
     /// See [conventional User ID] for more information.
     ///
     ///   [conventional User ID]: #conventional-user-ids
+    pub fn name2(&self) -> Result<Option<&str>> {
+        Ok(self.do_parse()?.name())
+    }
+
+    /// Parses the User ID according to de facto conventions, and
+    /// returns the name component, if any.
+    ///
+    /// Like [`UserID::name2`], but heap-allocates.
+    #[deprecated(note = "Use UserID::name2")]
     pub fn name(&self) -> Result<Option<String>> {
-        self.do_parse()?;
-        match *self.parsed.lock().unwrap().borrow() {
-            Some(ref puid) => Ok(puid.name().map(|s| s.to_string())),
-            None => unreachable!(),
-        }
+        Ok(self.do_parse()?.name().map(|s| s.to_string()))
     }
 
     /// Parses the User ID according to de facto conventions, and
@@ -881,12 +914,17 @@ impl UserID {
     /// See [conventional User ID] for more information.
     ///
     ///   [conventional User ID]: #conventional-user-ids
+    pub fn comment2(&self) -> Result<Option<&str>> {
+        Ok(self.do_parse()?.comment())
+    }
+
+    /// Parses the User ID according to de facto conventions, and
+    /// returns the comment field, if any.
+    ///
+    /// Like [`UserID::comment2`], but heap-allocates.
+    #[deprecated(note = "Use UserID::comment2")]
     pub fn comment(&self) -> Result<Option<String>> {
-        self.do_parse()?;
-        match *self.parsed.lock().unwrap().borrow() {
-            Some(ref puid) => Ok(puid.comment().map(|s| s.to_string())),
-            None => unreachable!(),
-        }
+        Ok(self.do_parse()?.comment().map(|s| s.to_string()))
     }
 
     /// Parses the User ID according to de facto conventions, and
@@ -895,12 +933,17 @@ impl UserID {
     /// See [conventional User ID] for more information.
     ///
     ///   [conventional User ID]: #conventional-user-ids
+    pub fn email2(&self) -> Result<Option<&str>> {
+        Ok(self.do_parse()?.email())
+    }
+
+    /// Parses the User ID according to de facto conventions, and
+    /// returns the email address, if any.
+    ///
+    /// Like [`UserID::email2`], but heap-allocates.
+    #[deprecated(note = "Use UserID::email2")]
     pub fn email(&self) -> Result<Option<String>> {
-        self.do_parse()?;
-        match *self.parsed.lock().unwrap().borrow() {
-            Some(ref puid) => Ok(puid.email().map(|s| s.to_string())),
-            None => unreachable!(),
-        }
+        Ok(self.do_parse()?.email().map(|s| s.to_string()))
     }
 
     /// Parses the User ID according to de facto conventions, and
@@ -909,12 +952,17 @@ impl UserID {
     /// See [conventional User ID] for more information.
     ///
     ///   [conventional User ID]: #conventional-user-ids
+    pub fn uri2(&self) -> Result<Option<&str>> {
+        Ok(self.do_parse()?.uri())
+    }
+
+    /// Parses the User ID according to de facto conventions, and
+    /// returns the URI, if any.
+    ///
+    /// Like [`UserID::uri2`], but heap-allocates.
+    #[deprecated(note = "Use UserID::uri2")]
     pub fn uri(&self) -> Result<Option<String>> {
-        self.do_parse()?;
-        match *self.parsed.lock().unwrap().borrow() {
-            Some(ref puid) => Ok(puid.uri().map(|s| s.to_string())),
-            None => unreachable!(),
-        }
+        Ok(self.do_parse()?.uri().map(|s| s.to_string()))
     }
 
     /// Returns a normalized version of the UserID's email address.
@@ -936,10 +984,9 @@ impl UserID {
     ///   [empty locale]: https://www.w3.org/International/wiki/Case_folding
     ///   [Autocrypt]: https://autocrypt.org/level1.html#e-mail-address-canonicalization
     pub fn email_normalized(&self) -> Result<Option<String>> {
-        match self.email() {
-            e @ Err(_) => e,
-            Ok(None) => Ok(None),
-            Ok(Some(address)) => {
+        match self.email2()? {
+            None => Ok(None),
+            Some(address) => {
                 let mut iter = address.split('@');
                 let localpart = iter.next().expect("Invalid email address");
                 let domain = iter.next().expect("Invalid email address");
@@ -947,8 +994,8 @@ impl UserID {
 
                 // Normalize Unicode in domains.
                 let domain = idna::domain_to_ascii(domain)
-                    .map_err(|e| anyhow::anyhow!(
-                        "punycode conversion failed: {:?}", e))?;
+                    .map_err(|e| anyhow::Error::from(e)
+                             .context("punycode conversion failed"))?;
 
                 // Join.
                 let address = format!("{}@{}", localpart, domain);
@@ -1003,6 +1050,10 @@ mod tests {
              email: Option<&str>, uri: Option<&str>)
             -> bool
         {
+            assert!(email.is_none() || uri.is_none());
+            t!("userid: {}, name: {:?}, comment: {:?}, email: {:?}, uri: {:?}",
+               userid, name, comment, email, uri);
+
             match ConventionallyParsedUserID::new(userid) {
                 Ok(puid) => {
                     let good = puid.name() == name
@@ -1258,6 +1309,39 @@ mod tests {
         }
     }
 
+    #[test]
+    fn compose() {
+        tracer!(true, "compose", 0);
+
+        fn c(userid: &str,
+             name: Option<&str>, comment: Option<&str>,
+             email: Option<&str>, uri: Option<&str>)
+        {
+            assert!(email.xor(uri).is_some());
+            t!("userid: {}, name: {:?}, comment: {:?}, email: {:?}, uri: {:?}",
+               userid, name, comment, email, uri);
+
+            if let Some(email) = email {
+                let uid = UserID::from_address(name, comment, email).unwrap();
+                assert_eq!(userid, String::from_utf8_lossy(uid.value()));
+            }
+
+            if let Some(uri) = uri {
+                let uid =
+                    UserID::from_unchecked_address(name, comment, uri).unwrap();
+                assert_eq!(userid, String::from_utf8_lossy(uid.value()));
+            }
+        }
+
+        // Conventional User IDs:
+        c("First Last (Comment) <name@example.org>",
+          Some("First Last"), Some("Comment"), Some("name@example.org"), None);
+        c("First Last <name@example.org>",
+          Some("First Last"), None, Some("name@example.org"), None);
+        c("<name@example.org>",
+          None, None, Some("name@example.org"), None);
+    }
+
     // Make sure we can't parse non conventional User IDs.
     #[test]
     fn decompose_non_conventional() {
@@ -1350,7 +1434,7 @@ mod tests {
     fn from_address() {
         assert_eq!(UserID::from_address(None, None, "foo@bar.com")
                        .unwrap().value(),
-                   b"foo@bar.com");
+                   b"<foo@bar.com>");
         assert!(UserID::from_address(None, None, "foo@@bar.com").is_err());
         assert_eq!(UserID::from_address("Foo Q. Bar".into(), None, "foo@bar.com")
                       .unwrap().value(),
