@@ -120,6 +120,17 @@ impl fmt::Debug for PacketPile {
 }
 
 impl<'a> Parse<'a, PacketPile> for PacketPile {
+    /// Deserializes the OpenPGP message stored in the file named by
+    /// `path`.
+    ///
+    /// See `from_reader` for more details and caveats.
+    fn from_buffered_reader<R>(reader: R) -> Result<PacketPile>
+    where
+        R: BufferedReader<Cookie> + 'a,
+    {
+        PacketPile::from_cookie_reader(reader.into_boxed())
+    }
+
     /// Deserializes the OpenPGP message stored in a `std::io::Read`
     /// object.
     ///
@@ -136,7 +147,7 @@ impl<'a> Parse<'a, PacketPile> for PacketPile {
     fn from_reader<R: 'a + io::Read + Send + Sync>(reader: R) -> Result<PacketPile> {
         let bio = buffered_reader::Generic::with_cookie(
             reader, None, Cookie::default());
-        PacketPile::from_buffered_reader(Box::new(bio))
+        PacketPile::from_cookie_reader(Box::new(bio))
     }
 
     /// Deserializes the OpenPGP message stored in the file named by
@@ -144,7 +155,7 @@ impl<'a> Parse<'a, PacketPile> for PacketPile {
     ///
     /// See `from_reader` for more details and caveats.
     fn from_file<P: AsRef<Path>>(path: P) -> Result<PacketPile> {
-        PacketPile::from_buffered_reader(
+        PacketPile::from_cookie_reader(
             Box::new(buffered_reader::File::with_cookie(path, Cookie::default())?))
     }
 
@@ -154,7 +165,7 @@ impl<'a> Parse<'a, PacketPile> for PacketPile {
     fn from_bytes<D: AsRef<[u8]> + ?Sized>(data: &'a D) -> Result<PacketPile> {
         let bio = buffered_reader::Memory::with_cookie(
             data.as_ref(), Cookie::default());
-        PacketPile::from_buffered_reader(Box::new(bio))
+        PacketPile::from_cookie_reader(Box::new(bio))
     }
 }
 
@@ -504,9 +515,9 @@ impl PacketPile {
     }
 
 
-    pub(crate) fn from_buffered_reader<'a>(bio: Box<dyn BufferedReader<Cookie> + 'a>)
+    pub(crate) fn from_cookie_reader<'a>(bio: Box<dyn BufferedReader<Cookie> + 'a>)
             -> Result<PacketPile> {
-        PacketParserBuilder::from_buffered_reader(bio)?
+        PacketParserBuilder::from_cookie_reader(bio)?
             .buffer_unread_content()
             .into_packet_pile()
     }
@@ -516,8 +527,18 @@ impl From<Cert> for PacketPile {
     /// Converts the `Cert` into a `PacketPile`.
     ///
     /// If any packets include secret key material, that secret key
-    /// material is not dropped, as it is when serializing a `Cert`.
+    /// material is included in the resulting `PacketPile`.  In
+    /// contrast, when serializing a `Cert`, or converting a cert to
+    /// packets with [`Cert::into_packets2`], the secret key material
+    /// not included.
+    ///
+    /// Note: This will change in sequoia-openpgp version 2, which
+    /// will harmonize the behavior and not include secret key
+    /// material.
+    // XXXv2: Drop the note in the doc comment and mentioned it in the
+    // release notes.
     fn from(cert: Cert) -> PacketPile {
+        #[allow(deprecated)]
         PacketPile::from(cert.into_packets().collect::<Vec<Packet>>())
     }
 }
